# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-10-05 07:15
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('hidash', '0002_auto_20160929_1312'),
    ]

    operations = [
        migrations.AddField(
            model_name='scheduledreport',
            name='email_subject',
            field=models.CharField(default=b'', max_length=1000),
        ),
        migrations.AlterField(
            model_name='chartparameter',
            name='parameter_type',
            field=models.IntegerField(blank=True, choices=[(0, b'dropdown'), (1, b'dateTime'), (2, b'dateRange')], null=True),
        ),
    ]
