# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-09-29 13:12
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('hidash', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ChartParameter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('parameter_name', models.CharField(blank=True, default=b'', max_length=128)),
                ('parameter_type', models.IntegerField(blank=True, choices=[(0, b'dropdown'), (1, b'dateTime')], null=True)),
                ('priority', models.PositiveIntegerField(blank=True, null=True)),
                ('grid_width', models.IntegerField(blank=True, choices=[(1, b'1'), (2, b'2'), (3, b'3'), (4, b'4'), (5, b'5'), (6, b'6'), (7, b'7'), (8, b'8'), (9, b'9'), (10, b'10'), (11, b'11'), (12, b'12')], null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Group',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=128)),
            ],
        ),
        migrations.CreateModel(
            name='ReportRecipients',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('email_address', models.CharField(blank=True, default=b'', max_length=512)),
            ],
        ),
        migrations.CreateModel(
            name='ScheduledReport',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
            ],
        ),
        migrations.CreateModel(
            name='ScheduledReportParam',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('parameter_name', models.CharField(blank=True, default=b'', max_length=128)),
                ('parameter_value', models.CharField(blank=True, default=b'', max_length=128)),
                ('scheduled_report', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='hidash.ScheduledReport')),
            ],
        ),
        migrations.RemoveField(
            model_name='chart',
            name='frequency',
        ),
        migrations.RemoveField(
            model_name='chart',
            name='send_report_in_email',
        ),
        migrations.RemoveField(
            model_name='chartgroup',
            name='name',
        ),
        migrations.AddField(
            model_name='chart',
            name='description',
            field=models.CharField(blank=True, max_length=2048, null=True),
        ),
        migrations.AddField(
            model_name='chart',
            name='grid_width',
            field=models.IntegerField(blank=True, choices=[(1, b'1'), (2, b'2'), (3, b'3'), (4, b'4'), (5, b'5'), (6, b'6'), (7, b'7'), (8, b'8'), (9, b'9'), (10, b'10'), (11, b'11'), (12, b'12')], null=True),
        ),
        migrations.AddField(
            model_name='chart',
            name='height',
            field=models.CharField(blank=True, max_length=10, null=True),
        ),
        migrations.AddField(
            model_name='chart',
            name='priority',
            field=models.PositiveIntegerField(blank=True, null=True),
        ),
        migrations.AddField(
            model_name='chart',
            name='separator',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AlterField(
            model_name='chart',
            name='dimension',
            field=models.CharField(blank=True, max_length=128, null=True),
        ),
        migrations.AlterField(
            model_name='chart',
            name='library',
            field=models.IntegerField(blank=True, choices=[(0, b'highcharts'), (1, b'googlechart')], null=True),
        ),
        migrations.AlterField(
            model_name='chart',
            name='name',
            field=models.CharField(max_length=128, unique=True),
        ),
        migrations.AlterField(
            model_name='chartmetric',
            name='metric',
            field=models.CharField(max_length=128),
        ),
        migrations.AddField(
            model_name='scheduledreport',
            name='report',
            field=models.ManyToManyField(to='hidash.Chart'),
        ),
        migrations.AddField(
            model_name='reportrecipients',
            name='report',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='hidash.ScheduledReport'),
        ),
        migrations.AddField(
            model_name='chartparameter',
            name='group',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='hidash.Group'),
        ),
        migrations.AddField(
            model_name='chartgroup',
            name='group',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, to='hidash.Group'),
            preserve_default=False,
        ),
    ]

