# -*- coding: utf-8 -*-
# Generated by Django 1.9.7 on 2016-09-29 13:03
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Chart',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=25, unique=True)),
                ('library', models.CharField(max_length=15)),
                ('dimension', models.CharField(max_length=50)),
                ('chart_type', models.CharField(blank=True, max_length=15, null=True)),
                ('query', models.TextField(max_length=10000)),
                ('send_report_in_email', models.BooleanField(default=False)),
                ('frequency', models.IntegerField(blank=True, choices=[(0, b'Daily'), (1, b'Weekly'), (2, b'Monthly'), (3, b'Yearly')], null=True)),
            ],
        ),
        migrations.CreateModel(
            name='ChartAuthGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('auth_group', models.CharField(max_length=50)),
                ('chart', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='hidash.Chart')),
            ],
        ),
        migrations.CreateModel(
            name='ChartAuthPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('permission', models.CharField(max_length=50)),
                ('chart', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='hidash.Chart')),
            ],
        ),
        migrations.CreateModel(
            name='ChartGroup',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=25)),
                ('chart', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='hidash.Chart')),
            ],
        ),
        migrations.CreateModel(
            name='ChartMetric',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('metric', models.CharField(max_length=50)),
                ('chart', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='chart', to='hidash.Chart')),
            ],
        ),
    ]

